#ifndef _LINUX_FS_STRUCT_H
#define _LINUX_FS_STRUCT_H

struct dentry;
struct vfsmount;

struct fs_struct {
	atomic_t count;
	rwlock_t lock;
	int umask;
	struct dentry * root, * pwd, * altroot;
	struct vfsmount * rootmnt, * pwdmnt, * altrootmnt;

	int flags;
	/* shadow dirs: root and pwd */
	/* | shdwroot | shdwrootmnt | result
	   +----------+-------------+------------
	   | NULL     | NULL        | BUG_ON(flags&SHDW_ENABLED)
	   | !NULL    | !NULL       | ok
	   +==========+=============+============
	   | shdwpwd  | shdwpwdmnt  | result
	   +----------+-------------+------------
	   | NULL     | NULL        | delayed
	   | NULL     | !NULL       | invalid
	   | !NULL    | NULL        | BUG
	   | !NULL    | !NULL       | valid */
	struct dentry *shdwroot, *shdwpwd;
	struct vfsmount *shdwrootmnt, *shdwpwdmnt;
	/* shadow dirs: escape character */
	unsigned char shdw_escch;
};

/* bitflags for fs_struct.flags */
#define SHDW_ENABLED		1	/* are shadow dirs enabled? */
#define SHDW_USE_ESC		2	/* use escape char in shadow dirs? */


#define INIT_FS {				\
	.count		= ATOMIC_INIT(1),	\
	.lock		= RW_LOCK_UNLOCKED,	\
	.umask		= 0022, \
}

extern struct kmem_cache *fs_cachep;

extern void exit_fs(struct task_struct *);
extern void set_fs_altroot(void);
extern void set_fs_root(struct fs_struct *, struct vfsmount *, struct dentry *);
extern void set_fs_pwd(struct fs_struct *, struct vfsmount *, struct dentry *);
extern void set_fs_shdwpwd(struct fs_struct *fs,
			   struct vfsmount *mnt, struct dentry *dentry);
extern struct fs_struct *copy_fs_struct(struct fs_struct *);
extern void put_fs_struct(struct fs_struct *);

#endif /* _LINUX_FS_STRUCT_H */
